
from .filler import FillerModel
from .template_name import TemplateNameModel
from .shared import TagModel
from .template import TemplateModel
from .template import LabelModel


__all__ = [
    "FillerModel",
    "TemplateNameModel",
    "TagModel",
    "TemplateModel",
    "LabelModel",
]



from typing import Dict
from typing import Any
from typing import Optional

import json
from dataclasses import dataclass


class RunnerBase:
    def __init__(self, json_string: Optional[str] = None):
        self.model = None

        if json_string:
            json_dict = json.loads(json_string)
            self.model = self.build(json_dict)

    def build(self, json_dict: Dict[str, Any]) -> dataclass:
        return NotImplementedError()

    def execute(self) -> None:
        raise NotImplementedError()




from thumbny.base import CommandBase
from thumbny.models import TemplateModel


class CreateCommand(CommandBase):
    def __init__(self, model: TemplateModel):
        super().__init__()
        self.model = model

    def execute(self):
        self.tm.create(self.model)
        print(f"{self.model.key} template has been created successfully")



from typing import Dict
from typing import Any
from typing import Optional

from thumbny.base import RunnerBase
from thumbny.commands import DeleteCommand
from thumbny.models import TemplateNameModel


class DeleteRunner(RunnerBase):
    def __init__(self, json_string: Optional[str] = None) -> None:
        super().__init__(json_string)

    def build(self, json_dict: Dict[str, Any]) -> TemplateNameModel:
        return TemplateNameModel.make(json_dict)

    def execute(self) -> None:
        command = DeleteCommand(self.model)
        return command.execute()



from typing import TYPE_CHECKING
from typing import List

import os
import re
import json
import shutil
import requests
from uuid import uuid4
from dataclasses import asdict

from thumbny.models import TemplateModel
from thumbny.exceptions import TemplateNotExist

if TYPE_CHECKING:
    from thumbny.templates_manager import TemplateManager


FILE_NAME_REGEX = r'[^\\|^/]+$'
LINK_REGEX = r'^http.+'


class FileHandler:
    def __init__(self,
                 template_manager: "TemplateManager",
                 templates_path: str) -> None:
        self.template_manager = template_manager
        self.templates_path = templates_path

    def create_template_dir(self, templates_path: str) -> None:
        if not os.path.exists(templates_path):
            os.makedirs(templates_path)

    def create_template_structure(self, key: str) -> str:
        template_path = os.path.join(self.templates_path, key)
        os.mkdir(template_path)
        os.mkdir(os.path.join(template_path, "assets"))
        os.mkdir(os.path.join(template_path, "assets", "fonts"))
        os.mkdir(os.path.join(template_path, "assets", "images"))
        return template_path

    def _copy_fonts(self, model: TemplateModel, template_path: str) -> None:
        for label in model.labels:
            if label.font_family:
                font_name = re.search(FILE_NAME_REGEX,
                                      label.font_family).group(0)

                font_path = os.path.join(template_path,
                                         "assets",
                                         "fonts",
                                         font_name)

                shutil.copyfile(label.font_family, font_path)
                label.font_family = font_path

    def _copy_images(self, model: TemplateModel, template_path: str) -> None:
        if re.match(LINK_REGEX, model.background_image):
            image = requests.get(model.background_image)
            image_name = str(uuid4())

            image_path = os.path.join(template_path,
                                      "assets",
                                      "images",
                                      image_name)

            with open(image_path, "wb") as file:
                file.write(image.content)

            model.background_image = image_path

        elif os.path.isfile(model.background_image):
            image_name = re.search(FILE_NAME_REGEX,
                                   model.background_image).group(0)

            image_path = os.path.join(template_path,
                                      "assets",
                                      "images",
                                      image_name)
            shutil.copyfile(model.background_image, image_path)
            model.background_image = image_path

    def copy_assets(self, model: TemplateModel, template_path: str) -> None:
        self._copy_fonts(model, template_path)
        self._copy_images(model, template_path)

    def save_config(self, config: TemplateModel, template_path: str) -> None:
        config_path = os.path.join(template_path, "config.json")
        with open(config_path, "w") as f:
            json.dump(asdict(config), f, indent=4)

    def delete_template(self, name: str) -> None:
        try:
            path = os.path.join(self.templates_path, name)
            shutil.rmtree(path)
        except FileNotFoundError:
            raise TemplateNotExist(f"{name} template does not exist")

    def get_all_templates(self) -> List[str]:
        if not os.path.exists(self.templates_path):
            return []

        return [element for element in os.listdir(self.templates_path)
                if os.path.isdir(os.path.join(self.templates_path, element))]

    def get_template_info(self, name: str) -> dict:
        path = os.path.join(self.templates_path, name, "config.json")
        if not os.path.isfile(path):
            raise TemplateNotExist(f"{name} template does not exist")
        with open(path) as f:
            return json.load(f)



from typing import TYPE_CHECKING

import os

from thumbny.exceptions import TemplateExist
from thumbny.exceptions import FontNotFound
from thumbny.exceptions import NotValidFontExtension


if TYPE_CHECKING:
    from thumbny.templates_manager import TemplateManager


class Validator:
    def __init__(self,
                 template_manager: "TemplateManager",
                 templates_path: str) -> None:
        self.template_manager = template_manager
        self.templates_path = templates_path

    def validate_tempalate_key(self, name: str) -> None:
        if name in self.template_manager.get_all_templates():
            raise TemplateExist(f"{name} template already exists")

    def validate_font_family(self, font_family: str) -> None:
        if font_family and not os.path.isfile(font_family):
            raise FontNotFound("Font not found")
        if font_family and not font_family.endswith("ttf"):
            raise NotValidFontExtension("Only ttf extension is supported")



from thumbny.templates_manager import TemplateManager
from thumbny.base import CommandBase


class TemplatesCommand(CommandBase):
    def __init__(self) -> None:
        self.template_manager = TemplateManager()

    def execute(self) -> None:
        templates = self.template_manager.get_all_templates()
        if templates:
            print("List of templates:")
            for x, y in enumerate(templates):
                print(f"{x+1}. {y}")
        else:
            print("No templates have been found")



from typing import Tuple
from typing import Optional

from thumbny.base import CommandBase
from thumbny.models import TemplateModel
from thumbny.models import LabelModel
from thumbny.models import FillerModel
from thumbny.enums import PositionTypeEnum
from thumbny.enums import XPositionEnum
from thumbny.enums import YPositionEnum


from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont


class GenerateCommand(CommandBase):
    def __init__(self, model: FillerModel) -> None:
        super().__init__()
        self.model = model

    def _hex_to_rgb(self, hex_color: str) -> Tuple[int]:
        hex_color = hex_color.lstrip('#')
        return tuple(int(hex_color[i:i + 2], 16) for i in (0, 2, 4))

    def _get_filename(self) -> str:
        return self.model.name.replace(" ", "_")

    def _get_template(self) -> TemplateModel:
        template = self.tm.get_template_info(self.model.template_key)
        return TemplateModel.make(template)

    def _find_template(self,
                       template: TemplateModel,
                       label_key: str) -> Optional[LabelModel]:
        for template_label in template.labels:
            if template_label.key == label_key:
                return template_label
        return None

    def execute(self) -> None:
        filename = self._get_filename()
        template = self._get_template()

        image = Image.new(mode="RGB",
                          size=(template.width, template.height),
                          color=template.background_color)

        draw = ImageDraw.Draw(image)

        if template.background_image:
            background = Image.open(template.background_image)
            image.paste(background, (0, 0))

        for label in self.model.labels:
            template_label = self._find_template(template, label.key)

            if template_label.font_family:
                try:
                    font = ImageFont.truetype(template_label.font_family,
                                              size=template_label.font_size)
                except OSError:
                    print(f"Font wasn't found at {template_label.font_family}")
            else:
                font = ImageFont.load_default(size=template_label.font_size)

            width = draw.textlength(label.value, font=font)
            height = template_label.font_size

            if template_label.position.key == PositionTypeEnum.RELATIVE.value:
                x_positions = {
                    XPositionEnum.LEFT.value: 0,
                    XPositionEnum.CENTER.value: (template.width - width) / 2,
                    XPositionEnum.RIGHT.value: template.width - width
                }

                y_positions = {
                    YPositionEnum.TOP.value: 0,
                    YPositionEnum.CENTER.value: (template.height - height) / 2,
                    YPositionEnum.BOTTOM.value: template.height - height
                }

                x_pos_key, y_pos_key = template_label.position.value.split(",")
                text_x = x_positions.get(x_pos_key, 0)
                text_y = y_positions.get(y_pos_key, 0)

            draw.text(xy=(text_x, text_y),
                      text=label.value,
                      fill=template_label.font_color,
                      font=font)

        image.save(f"{filename}.png")
        image.show()

        print("The thumbnail has been created successfully")



from typing import Dict
from typing import Any
from typing import Optional

from thumbny.base import RunnerBase
from thumbny.commands import CreateCommand
from thumbny.models import TemplateModel


class CreateRunner(RunnerBase):
    def __init__(self, json_string: Optional[str] = None) -> None:
        super().__init__(json_string)

    def build(self, json_dict: Dict[str, Any]) -> TemplateModel:
        return TemplateModel.make(json_dict)

    def execute(self) -> None:
        command = CreateCommand(self.model)
        command.execute()



from __future__ import annotations

from typing import Dict
from typing import Any

from dataclasses import dataclass


@dataclass
class TemplateNameModel:
    name: str

    @classmethod
    def make(cls, data: Dict[str, Any]) -> TemplateNameModel:
        return cls(name=data.get("name"))



from __future__ import annotations

from typing import Dict
from typing import Any

from dataclasses import dataclass

from thumbny.models.validation import check_required_fields


@dataclass
class TagModel:
    key: str
    value: str

    def __post_init__(self):
        check_required_fields(self)

    @classmethod
    def make(cls, data: Dict[str, Any]) -> TagModel:
        return cls(key=data.get("key"),
                   value=data.get("value"))



from __future__ import annotations

from typing import List
from typing import Dict
from typing import Any
from dataclasses import dataclass

from thumbny.models.shared import TagModel
from thumbny.models.validation import check_required_fields


@dataclass
class FillerModel:
    name: str
    template_key: str
    labels: List[TagModel]

    def __post_init__(self):
        check_required_fields(self)

    @classmethod
    def make(cls, data: Dict[str, Any]) -> FillerModel:
        labels = []
        for label in data.get("labels", []):
            labels.append(TagModel.make(label))

        return cls(name=data.get("name"),
                   template_key=data.get("template_key"),
                   labels=labels)



from thumbny.base import CommandBase
from thumbny.models import TemplateNameModel


class DeleteCommand(CommandBase):
    def __init__(self, model: TemplateNameModel) -> None:
        super().__init__()
        self.model = model

    def execute(self) -> None:
        self.tm.delete(self.model.name)
        print(f"{self.model.name} template has been deleted successfully")



from typing import Dict
from typing import Any

from thumbny.base import CommandBase
from thumbny.models import TemplateNameModel


class InfoCommand(CommandBase):
    def __init__(self, model: TemplateNameModel) -> None:
        super().__init__()
        self.model = model

    def _print_info(self,
                    details: Dict[str, Any],
                    post_indent: int = 0,
                    indent: int = 20) -> None:
        for x, y in details.items():
            if isinstance(y, list):
                for record in details[x]:
                    self._print_info(record, post_indent+5, indent)
                return None
            elif isinstance(y, dict):
                self._print_info(y, post_indent+5, indent)
                return None
            else:
                print(f"{'':<{post_indent}} {x:<{indent}} {y}")

    def execute(self) -> None:
        details = self.tm.get_template_info(self.model.name)
        print("="*40)
        print(f"{'Key':<20} {'Value'}")
        print("="*40)
        self._print_info(details)
